@@ -1,12 +1,14 @@
 ### FUNCTIONS FOR HIDDEN MARKOV MODELS IN CONTINUOUS TIME
 ### WITH ARBITRARY RESPONSE DISTRIBUTION
 
+#' @export
 print.hmmMVdist <- function(x, ...)
 {
     cat(sprintf("Multivariate hidden Markov model with %d outcomes:\n", length(x)))
     for (i in x) print(i)
 }
 
+#' @export
 print.hmmdist <- function(x, ...)
 {
     cat("Hidden Markov model", x$label, "distribution\n\n")
@@ -18,7 +20,6 @@ print.hmmdist <- function(x, ...)
 
 msm.check.hmodel <- function(hmodel, nstates)
   {
-      if (is.null(hmodel)) stop("Hidden model not specified")
       if (!is.list(hmodel)) stop("Hidden model should be a list")
       if (length(hmodel) != nstates) stop("hmodel of length ", length(hmodel), ", expected ", nstates)
       for (i in hmodel) {
@@ -276,6 +277,7 @@ msm.econstr2hconstr <- function(econstr, hmodel)
       match(constr, unique(constr))
   }
 
+#' @export
 print.hmodel <- function(x, ...)
   {
       ci <- (x$fitted && x$foundse)
@@ -306,7 +308,7 @@ print.hmodel <- function(x, ...)
                   for (j in 1:x$nout[i]){
                       cat("Outcome", j, "-", x$labels[j,i], "distribution\n")
                       if (x$labels[j,i]=="categorical")
-                          pars <- print.hmmcat(x, i, j, mv=TRUE)   ## TESTME
+                          pars <- print_hmmcat(x, i, j, mv=TRUE)   ## TESTME
                       else {
                           inds <- x$parstate==i & x$parout==j
                           pars <- as.matrix(x$pars[inds])
@@ -321,7 +323,7 @@ print.hmodel <- function(x, ...)
                   cat("State", i, "-", x$labels[i], "distribution\n")
                   cat("Parameters: \n")
                   if (x$label[i]=="categorical")
-                      pars <- print.hmmcat(x, i)
+                      pars <- print_hmmcat(x, i)
                   else {
                       pars <- as.matrix(x$pars[x$parstate==i])
                       if (ci) pars <- cbind(pars, matrix(x$ci[x$parstate==i ,], ncol=2))
@@ -340,7 +342,7 @@ print.hmodel <- function(x, ...)
       }
   }
 
-print.hmmcat <- function(x, i, j=NULL, mv=FALSE)
+print_hmmcat <- function(x, i, j=NULL, mv=FALSE)
 {
     inds <- if (mv) x$parstate==i & x$parout==j else x$parstate==i
     pars <- x$pars[inds]
